#!/usr/local/gnu/bin/gawk -f
# Adjust the above line to point to _your_ gawk executable, or be
# prepared to deal with _really_ weird error messages.
# 
# Programm otter_to_discount.awk
#
# Translates (tptp2X-generated) Otter input files into DISCOUNT input
# files. Expects 'equal(t1,t2)' to be used for equations, not 
# 't1 = t2'. Interprets symbols that start with a capital letter as
# variables, all other symbols as function symbols. Otter input files
# should have _all_ equality axioms removed
# (i.e. -t remove_equality:rstfp).  
#
# Usage: otter_to_discount.awk otter_file > discount_file
#
# Stephan Schulz, 24.6.1997

BEGIN{
  out_eqns = "";
  max_arity = 0;
  paramod = "";
  in_goal = 0;
  goal = "";
}

function max(x,y)
{
  if(x>y)
  {
    return x;
  }
  return y;
}

function SigInsert(symbol,arity)
{
  if(symbol in signature)
  {
    if(signature[symbol]!=arity)
    {
      print "Symbol " symbol " has conflicting arities" > "/dev/stderr";
      exit 1;
    }
  }
  else if(symbol in g_signature)
  {
    if(g_signature[symbol]!=arity)
    {
      print "Symbol " symbol " has conflicting arities" > "/dev/stderr";
      exit 1;
    }
  }
  else
  {
    if(!in_goal)
    {
      signature[symbol] = arity;
    }
    else
    {
      g_signature[symbol] = arity;
    }
    max_arity = max(max_arity,arity);
  }
}

function ParseArgList( res, symbol,    i)
{
  i = 0;
  $0 = substr($0,2);
  res = res "(";
  res = ParseTerm(res);
  while(index($0,",")==1)
  {
    i++;
    res = res ",";
    $0 = substr($0,2);
    res = ParseTerm(res);
  }
  if(index($0,")")!=1)
  {
    print "Bracket not closed properly!" > "/dev/stderr";
    exit 1;
  }
  res = res ")";
  $0 = substr($0,2);
  SigInsert(symbol,i);
  return res;
}


function ParseTerm(res    ,symbol)
{
  if(match($0,/[A-Z][A-Za-z0-9_]*/)==1)  # Variable 
  {
    # print "Variable detected";
    res = res substr($0,1,RLENGTH);
    $0 = substr($0,RLENGTH+1);
  }
  else if(match($0,/[a-z0-9_]+/)==1) # Function symbol 
  {
    symbol = substr($0,1,RLENGTH);
    # print "Symbol is " symbol;
    res = res symbol;
    $0 = substr($0,RLENGTH+1);
    if(index($0,"(")==1)
    {
      res = ParseArgList(res, symbol);
    }
    else
    {
      SigInsert(symbol,0);
    }
  }
  return res;
}

function GenOrdArity(sig, i,   preord)
{
  preord = "";
  
  for(j in sig)
  {
    if(sig[j] == i)
    {
      preord = preord " > " j ":1";
    }
  }
  return preord;
}   

function GenOrd(    i, j, preord)
{
  preord = GenOrdArity(signature, 1);

  for(i=max_arity;i>1;i--)
  {
    preord = preord GenOrdArity(signature, i);
  }
  preord = preord GenOrdArity(signature, 0);
  
  preord = preord GenOrdArity(g_signature, 1);

  for(i=max_arity;i>1;i--)
  {
    preord = preord GenOrdArity(g_signature, i);
  }
  preord = preord GenOrdArity(g_signature, 0);
  
  preord = substr(preord,4);
  return preord;
}

/%/{
  # _Really_ Ignore comments (and ensure
  # that further rules cannot cause havoc)
  $0 = substr($0,1,index($0,"%"));
}

/equal/ && !/-equal/{
  # print "Aktuelle Zeile:" $0;
  gsub(/ /, ""); # Get rid of spaces
  $0 = substr($0,index($0,"(")+1);
  equation = ParseTerm("");
  $0 = substr($0,index($0,",")+1);
  equation = equation " = ";
  equation = equation ParseTerm("");
  outeqns = outeqns "           " equation "\n";
  
}

/-equal/{
  in_goal = 1;
  gsub(/ /, ""); # Get rid of spaces
  $0 = substr($0,index($0,"(")+1);
  goal = goal ParseTerm("");
  $0 = substr($0,index($0,",")+1);
  goal = goal " = ";
  goal = goal ParseTerm("");
  if(match(goal,/[A-Z][A-Za-z0-9_]*/))
  {
    paramod = ", PARAMOD(2)";
  }
  goal = "CONCLUSION " goal;
}
    
END{
  print "MODE PROOF" paramod "\n";
  gsub(/.*\//,"",ARGV[1]);
  gsub(/[-+.]/,"_",ARGV[1]);
  print "NAME " ARGV[1] "\n";
  print "ORDERING XKBO";
  print "           " GenOrd();
  print "";
  print "EQUATIONS";
  print outeqns;
  print goal;
}
