%###########################################################################
%# write_lop.pl
%# - formatted output of LOP clauses, rules and comments
%# - comments:	comment('bla bla').
%# - clauses:	clause( Name, Type, LitList, Cons, Dict ) 
%# - rules:	rule( Name, Type, LitList, Cons, Dict ) 
%###########################################################################
%# predicates to be used from outside:
%#	- printout/1
%#	- printout_list/1
%# dependent on
%#	- mylib.pl
%###########################################################################
%#        Max Moser                         Automated Reasoning Research Group
%# Email: moser@informatik.tu-muenchen.de     Lehrstuhl VIII f"ur Informatik
%# Tel.:  +49-89/521096			     Technische Universit"at M"unchen
%# Fax.:  +49-89/526502			         D-80290 M"unchen, Germany
%# http://wwwjessen.informatik.tu-muenchen.de/~moser
%###########################################################################


%*****************************************************************************
%* printout_lop( +ClauseOrRuleOrComment )
%* printout_lop_list( +ClauseOrRuleOrCommentList )
%*****************************************************************************
%* - print description, instantiate dictionary, and print clause or rule
%*****************************************************************************

printout_lop( comment( Comment ) ) :-
	write( '# ' ), write( Comment ), writeln( "\n" ).

printout_lop( nomodclause( Name, Type, LitList, Cons, Dict ) ) :-
	printout_lop( clause( Name, Type, LitList, Cons, Dict ) ).

printout_lop( nomodrule( Name, Type, LitList, Cons, Dict ) ) :-
	printout_lop( rule( Name, Type, LitList, Cons, Dict ) ).

printout_lop( clause( Name, Type, LitList, Cons, Dict ) ) :-
	write( '# ' ), write( Name ), write( ', ' ), write( Type ),
	writeln( '.' ),
	inst_dict( Dict ),
	litlist_to_posneglist( LitList, PosLits, NegLits ),
	print_clause( PosLits, NegLits, Cons ).

printout_lop( rule( Name, Type, LitList, Cons, Dict ) ) :-
	write( '# ' ), write( Name ), write( ', ' ), write( Type ),
	writeln( '.' ),
	inst_dict( Dict ),
	litlist_to_posneglist( LitList, PosLits, NegLits ),
	print_clause( PosLits, NegLits, Cons ).


printout_lop_list( [] ).

printout_lop_list( [ClauseOrRule|Rest] ) :-
	on_verbose_report( '.', "" ),
	printout_lop( ClauseOrRule ),
	printout_lop_list( Rest ).

%*****************************************************************************


%*****************************************************************************
%* print_clause( +Head, +Body, +Cons )
%*****************************************************************************
%* print clause
%*****************************************************************************

print_clause( [], Body, [] ) :-
	!,
	write( ' <- ' ),
	print_seq( Body, ',\n    ' ),
	write( '.' ), nl, nl.

print_clause( [], Body, Cons ) :-
	!,
	write( ' <- ' ),
	print_seq( Body, ',\n    ' ),
	write( ' :\n    ' ),
	print_seq_cons( Cons, ',\n    ' ),
	write( '.' ), nl, nl.

print_clause( Head, [], [] ) :-
	!,
	print_seq( Head, ";\n" ),
	write( ' <- ' ),
	write( '.' ), nl, nl.

print_clause( Head, [], Cons ) :-
	!,
	print_seq( Head, ";\n" ),
	write( ' <- ' ),
	write( ' :\n    ' ),
	print_seq_cons( Cons, ',\n    ' ),
	write( '.' ), nl, nl.

print_clause( Head, Body, [] ) :-
	!,
	print_seq( Head, ";\n" ),
	write( ' <-\n    ' ),
	print_seq( Body, ',\n    ' ),
	write( '.' ), nl, nl.

print_clause( Head, Body, Cons ) :-
	print_seq( Head, ";\n" ),
	write( ' <-\n    ' ),
	print_seq( Body, ',\n    ' ),
	write( ' :\n    ' ),
	print_seq_cons( Cons, ',\n    ' ),
	write( '.' ), nl, nl.

%*****************************************************************************


%*****************************************************************************
%* print_rule( +Head, +Body, +Cons )
%*****************************************************************************
%* print rule
%*****************************************************************************

print_rule( [], Body, [] ) :-
	!,
	write( ' ?- ' ),
	print_seq( Body, ',\n    ' ),
	write( '.' ), nl, nl.

print_rule( [], Body, Cons ) :-
	!,
	write( ' ?- ' ),
	print_seq( Body, ',\n    ' ),
	write( ' :\n    ' ),
	print_seq_cons( Cons, ',\n    ' ),
	write( '.' ), nl, nl.

print_rule( Head, [], [] ) :-
	!,
	print_seq( Head, ";\n" ),
	write( '.' ), nl, nl.

print_rule( Head, [], Cons ) :-
	!,
	print_seq( Head, ";\n" ),
	write( ' :\n    ' ),
	print_seq_cons( Cons, ',\n    ' ),
	write( '.' ), nl, nl.

print_rule( Head, Body, [] ) :-
	!,
	print_seq( Head, ";\n" ),
	write( ' :-\n    ' ),
	print_seq( Body, ',\n    ' ),
	write( '.' ), nl, nl.

print_rule( Head, Body, Cons ) :-
	print_seq( Head, ";\n" ),
	write( ' :-\n    ' ),
	print_seq( Body, ',\n    ' ),
	write( ' :\n    ' ),
	print_seq_cons( Cons, ',\n    ' ),
	write( '.' ), nl, nl.


print_seq_lit( (X is Y) ) :-
	!,
	printf("%Dw is %Dw",[X,Y]).
print_seq_lit( (~ Lit) ) :-
	!,
	printf("~ %ODw",Lit).
print_seq_lit( Lit ) :-
	printf("%ODw",Lit).

print_seq( [Lit], _ ) :-
	print_seq_lit( Lit ).
print_seq( [Lit1,Lit2|T], Del ) :-
	print_seq_lit( Lit1 ), write( Del ),
	print_seq( [Lit2|T], Del ).

print_seq_cons( [(L,R)], _ ) :-
	printf("%ODw=/=%ODw",[L,R]).
print_seq_cons( [(L,R),Cons2|T], Del ) :-
	printf("%ODw=/=%ODw",[L,R]), write( Del ),
	print_seq_cons( [Cons2|T], Del ).

%*****************************************************************************
