%-----------------------------------------------------------------------------
%- specify filter files to be consulted 
%-----------------------------------------------------------------------------

:- [mod_newswitches].
:- [mod_umod].
:- [mod_replnum].
:- [mod_rmeqax].
:- [mod_expose].
:- [mod_stemod].
:- [mod_rmod].
:- [mod_rel].
:- [mod_stat].
:- [mod_sig].
:- [mod_addord].
:- [mod_genord].
:- [mod_split].
:- [mod_ground].
:- [mod_demod].
:- [mod_isol].

%-----------------------------------------------------------------------------
%- configure procedures for processing switches
%-----------------------------------------------------------------------------

process_switch( Switch ) :-
	process_switch_umod( Switch );
	process_switch_replnum( Switch );
	process_switch_rmeqax( Switch );
        process_switch_demod( Switch );
	process_switch_expose( Switch );
        process_switch_stemod( Switch );
        process_switch_ceemod( Switch );
	process_switch_brandstemod( Switch );
	process_switch_moserstemod( Switch );
	process_switch_rmod( Switch );
	process_switch_rel( Switch );
	process_switch_stat( Switch );
	process_switch_addord( Switch );
        process_switch_sort( Switch );
        process_switch_noquerytrans( Switch );
        process_switch_psetheo( Switch );
        process_switch_sig( Switch );
        process_switch_split( Switch );
	process_switch_nosplitlits( Switch );
	process_switch_nogroundsplit( Switch );
	process_switch_isol( Switch );
	process_switch_ground( Switch ).

%-----------------------------------------------------------------------------
%- configure predicates for testing (and evtl resetting) switch combinations
%-----------------------------------------------------------------------------

test_switches :-
	test_switches_umod,
	test_switches_rmeqax,
	test_switches_demod,
	test_switches_expose,
	test_switches_stemod,
	test_switches_brandstemod,
        test_switches_moserstemod,
        test_switches_ceemod,
	test_switches_rmod,
	test_switches_rel,
        test_switches_genord,
        test_switches_addord.

%-----------------------------------------------------------------------------
%- configure ordering in which filter predicates are to be invoked
%-----------------------------------------------------------------------------

filterseq( [	
	problemwise(demod),		
	problemwise(isol),
	clausewise(umod), 
	clausewise(replnum), 
	clausewise(rmeqax),
	clausewise(expose),
	problemwise(addrefleq),
	clausewise(mmod),
	clausewise(remod),
	clausewise(smod),
	clausewise(snmod),
        problemwise(genord),
	clausewise(addord),
        clausewise(tmod),
        clausewise(b_tmod),
	clausewise(tpmod),
        clausewise(tpcmod),
	clausewise(tnmod),
        clausewise(tncmod),
	clausewise(emod),
	clausewise(b_emod),
	clausewise(mlmod),
	clausewise(rmod),
	problemwise(sig),
	problemwise(split),
	problemwise(rel),
	problemwise(ground),
	problemwise(stat)
] ).

%-----------------------------------------------------------------------------
%- configure usage text to be printed when program is called
%-----------------------------------------------------------------------------

usage_text( "\
*** stexposu\n\
*** Max Moser und\n\
*** Joachim Steinbach, Automated Reasoning Research Group, TUM (1998)\n\
*** usage: stexposu [SWITCHES] INFILE > OUTFILE\n\
*** INFILE:	legal LOP or TPTP file name (with extension) or - for stdin\n\
*** OUTFILE:	legal LOP or TPTP file name (with extension)\n\
*** SWITCHES:	-tptpin -tptpout\n\
***		-verbose\n\
***		-rmeqax {r|s|t|e}* -demod\n\
***             -stemod {s|t|e}* -brandstemod {s|t|e}*\n\
***             -moserstemod {s|tp|re|tn|ml}* -ceemod {m|re|s|sn|tpc|tnc}*\n\
***             {-sortfst [OPERATOR]|-sortsnd [OPERATOR]}*\n\
***             -umod [MAXLENGTH] -expose -rel -stat -sig {c|h}* -replnum\n\
***             -rmod [COUNT] -noquerytrans -addord -psetheo\n\
***             -split [Count] -nosplitlits -nogroundsplit [0|1|2]\n\
***             -isol -ground [NoConst]\
" ).

%-----------------------------------------------------------------------------
%- configure name of program
%-----------------------------------------------------------------------------

program_name( 'hpstexposu' ) :-
    get_flag(hostarch,"hp700_ux90"),
    !.

program_name( 'stexposu' ).


%-----------------------------------------------------------------------------
%- configure base name of TPTP
%-----------------------------------------------------------------------------

tptp_basename( '/group/TPTP/TPTP-v2.1.0/' ) :-
    get_flag(hostarch,"sparc_sunos5").

tptp_basename( '/usr/share/hpjessen1/AR.Work/TPTP/TPTP-v2.0.0/' ) :-
    get_flag(hostarch,"hp700_ux90").
