%-----------------------------------------------------------------------------
%- specify filter files to be consulted 
%-----------------------------------------------------------------------------

:- [mod_sorts].
:- [mod_umod].
:- [mod_replnum].
:- [mod_rmeqax].
:- [mod_expose].
:- [mod_stemod].
:- [mod_brandstemod].
:- [mod_moserstemod].
:- [mod_rmod].
:- [mod_rel].
:- [mod_stat].
:- [mod_sig].
:- [mod_addord].
:- [mod_genord].

%-----------------------------------------------------------------------------
%- configure procedures for processing switches
%-----------------------------------------------------------------------------

process_switch( Switch ) :-
	process_switch_umod( Switch );
	process_switch_replnum( Switch );
	process_switch_rmeqax( Switch );
	process_switch_expose( Switch );
        process_switch_stemod( Switch );
	process_switch_brandstemod( Switch );
	process_switch_moserstemod( Switch );
	process_switch_rmod( Switch );
	process_switch_rel( Switch );
	process_switch_stat( Switch );
	process_switch_addord( Switch );
        process_switch_sort( Switch );
        process_switch_sig( Switch ).

%-----------------------------------------------------------------------------
%- configure predicates for testing (and evtl resetting) switch combinations
%-----------------------------------------------------------------------------

test_switches :-
	test_switches_umod,
	test_switches_rmeqax,
	test_switches_expose,
	test_switches_stemod,
	test_switches_brandstemod,
        test_switches_moserstemod,
	test_switches_rmod,
	test_switches_rel,
        test_switches_genord,
        test_switches_addord.

%-----------------------------------------------------------------------------
%- configure ordering in which filter predicates are to be invoked
%-----------------------------------------------------------------------------

filterseq( [	
	clausewise(umod), 
	clausewise(replnum), 
	clausewise(rmeqax),
	clausewise(expose),
	problemwise(addrefleq),
	clausewise(smod),
	problemwise(b_addrefleq),
	clausewise(b_smod),
	problemwise(o_addrefleq),
	clausewise(o_smod),
        problemwise(genord),
	clausewise(addord),
	clausewise(tmod),
	clausewise(emod),
	clausewise(b_tmod),
	clausewise(b_emod),
	clausewise(o_tmod),
	clausewise(o_emod),
	clausewise(rmod),
	problemwise(stat),
	problemwise(sig),
	problemwise(rel)
] ).

%-----------------------------------------------------------------------------
%- configure usage text to be printed when program is called
%-----------------------------------------------------------------------------

usage_text( "\
*** stexposu\n\
*** Max Moser, Automated Reasoning Research Group, TUM (1997)\n\
*** usage: stexposu [SWITCHES] INFILE > OUTFILE\n\
*** INFILE:	legal LOP or TPTP file name (with extension) or - for stdin\n\
*** OUTFILE:	legal LOP or TPTP file name (with extension)\n\
*** SWITCHES:	-tptpin -tptpout\n\
***		-verbose\n\
***		-rmeqax {r|s|t|e}*\n\
***             -stemod {s|t|e}* -brandstemod {s|t|e}* -moserstemod {s|t|e}*\n\
***             -sort [OPERATOR]\n\
***             -umod [MAXLENGTH] -expose -rel -stat -sig {c|h}* -replnum\n\
***             -rmod [COUNT] -addord\
" ).

%-----------------------------------------------------------------------------
%- configure name of program
%-----------------------------------------------------------------------------

program_name( 'stexposu' ).

program_name( 'hpstexposu' ) :-
	get_flag(hostarch,"hp700_ux90").


%-----------------------------------------------------------------------------
%- configure base name of TPTP
%-----------------------------------------------------------------------------

tptp_basename( '/group/TPTP/TPTP-v2.0.0/' ) :-
	get_flag(hostarch,"sparc_sunos5").

tptp_basename( '/usr/share/hpjessen1/AR.Work/steinbac/TPTP-v1.2.1/' ) :-
	get_flag(hostarch,"hp700_ux90").
