%###########################################################################
%# main.pl
%# - main file for STEXPOSU
%###########################################################################
%#        Max Moser                         Automated Reasoning Research Group
%# Email: moser@informatik.tu-muenchen.de     Lehrstuhl VIII f"ur Informatik
%# Tel.:  +49-89/521096			     Technische Universit"at M"unchen
%# Fax.:  +49-89/526502			         D-80290 M"unchen, Germany
%# http://wwwjessen.informatik.tu-muenchen.de/~moser
%###########################################################################

:- set_flag(occur_check,on).

:- lib(lists).
:- lib(sorts).

:- op(100,fx,++).
:- op(100,fx,--).
:- op( 99,fx,~).

:- abolish_op(not,fy).

:- [utils].
:- [errorhander].
:- [cmd_line].
:- [read_lop].
:- [read_tptp].
:- [siginfo].
:- [proc_loop].
:- [write_lop].
:- [write_tptp].
:- [testord].
:- [test4eq].

:- [config].
	%	- modifications
	%	- filter sequence
	%	- command line argument processing
	%	- switch testing predicates
	%	- usage text


%*****************************************************************************
%* main
%*****************************************************************************
%* - main call
%* - parse command line arguments
%* - read the problem clauses from input file
%* - process them
%* - print out result
%*****************************************************************************

main :-
	process_cmd_line,
	on_verbose_report( "* reading input", "\n" ),
	read_problem( ClauseList ),
	on_verbose_report( "--- done reading input\n* processing input:", "\n" ),
	siginfo( ClauseList ),
	test_switches,
        process_problem( ClauseList, NewClauseList ),
	on_verbose_report( "--- done processing input\n* writing result", "\n" ),
        write_problem( NewClauseList ),
	on_verbose_report( "--- done writing result\n* end", "\n" ),
	exit(0).

main :-
	writeln( stderr, '*** unknown problem' ),
	exit(2).

%*****************************************************************************


%*****************************************************************************
%* read_problem( -ClauseList )
%*****************************************************************************
%* - read input clauses
%* - default is to read LOP syntax
%* - tptpin switch is given then read TPTP syntax
%*****************************************************************************

read_problem( ClauseList ) :-
	cmd_arg( tptpin, _ ),
	!,
	read_tptp_problem( ClauseList ),
	!.

read_problem( ClauseList ) :-
	read_lop_problem( ClauseList ),
	!.

%*****************************************************************************


%*****************************************************************************
%* write_problem( +ClauseList )
%*****************************************************************************
%* - write final set of clauses
%* - default is to write LOP syntax
%* - tptpout switch is given then write TPTP syntax
%*****************************************************************************

write_problem( ClauseList ) :-
	cmd_arg( tptpout, _ ),
	!,
	printout_tptp_list( ClauseList ).

write_problem( ClauseList ) :-
	printout_lop_list( ClauseList ).

%*****************************************************************************



%*****************************************************************************
%* usage
%*****************************************************************************
%* - print information about how to start the program
%*****************************************************************************

usage :-
	usage_text( Usage_Text ),
	writeln( stderr, '*******************************************************************************'),
	writeln( stderr, Usage_Text ),
	writeln( stderr, '*******************************************************************************').

%*****************************************************************************


%*****************************************************************************
%* on_verbose_report( +Term, +Delim )
%*****************************************************************************
%* - if verbose switch is set, then write String to error, else do nothing
%*****************************************************************************

on_verbose_report( Term, Delim ) :-
	cmd_arg( verbose, _ ),
	!,
	write( error, Term ),
	write( error, Delim ).

on_verbose_report( _, _ ).

%*****************************************************************************
