%###########################################################################
%# addord.pl
%# - add ordering built ins for positive equations
%###########################################################################
%# predicates to be used from outside:
%#	- addord/4
%#	- process_switch_addord/1
%#	- test_switches_addord/0
%# dependent on
%#	- mylib.pl
%#	- clause info facts generated by siginfo.pl
%###########################################################################
%#        Max Moser                         Automated Reasoning Research Group
%# Email: moser@informatik.tu-muenchen.de     Lehrstuhl VIII f"ur Informatik
%# Tel.:  +49-89/521096			     Technische Universit"at M"unchen
%# Fax.:  +49-89/526502			         D-80290 M"unchen, Germany
%# http://wwwjessen.informatik.tu-muenchen.de/~moser
%###########################################################################


%*****************************************************************************
%* addord( clausewise, +Clause, +SwitchList, -NewClauseList )
%*****************************************************************************
%* - add ordering BIPs
%*****************************************************************************

addord( clausewise, Clause, _, [NewCl] ) :-
        Clause =.. [ ClauseCategory, Name, Type, LitList, Dict, Cons ],
	!,
	setval(rescnt,0),
	add_ord_bips( LitList, NewLitList ),
        NewCl =.. [ ClauseCategory, Name, Type, NewLitList, Dict, Cons ].

addord( clausewise, Clause, _, [Clause] ).

%*****************************************************************************


%*****************************************************************************
%* add_ord_bips( +LitList, -NewLitList )
%*****************************************************************************
%* - add ordering constraints for all positive equations
%*****************************************************************************

add_ord_bips( [], [] ).

add_ord_bips( [++equal(L,R)|RestClause], ModClause ) :-
	L \== R,
	!,
	test_order( L, R, OrdResult ),
	gen_ord_pred( (L,R), OrdResult, OrdBips ),
	add_ord_bips( RestClause, ModRestClause ),
	append( [++equal(L,R)|OrdBips], ModRestClause, ModClause ).

add_ord_bips( [Lit|RestClause], [Lit|ModRestClause] ) :-
	add_ord_bips( RestClause, ModRestClause ).

%*****************************************************************************


%*****************************************************************************
%* gen_ord_pred( +Expr, +OrdResult, -OrdPreds )
%*****************************************************************************
%* - generate ordering predicate with the two terms of eq
%*****************************************************************************

gen_ord_pred( (L,R), 0, [--OrdPred] ) :-
	get_new_Res_atom( NewResAtom ),
	OrdPred =.. ['$lpo',L,R,NewResAtom].

gen_ord_pred( _, 1, [--NewResExpr] ) :-
	get_new_Res_atom( NewResAtom ),
	name( NewResAtom, ResList ),
	append( ResList, [32, 105, 115, 32, 49], NewList ),
	name( NewResExpr, NewList ).

gen_ord_pred( _, -1, [] ).

%*****************************************************************************


%*****************************************************************************
%* get_new_Res_atom( -NewAtom )
%* get_cur_Res_atom( -CurrentAtom )
%*****************************************************************************
%* - generate new atoms for variable names
%*****************************************************************************

get_new_Res_atom( NewAtom ) :-
	incval( rescnt ),
	get_cur_Res_atom( NewAtom ).

get_cur_Res_atom( Atom ) :-
	getval( rescnt, ResCnt ),
	name( ResCnt, DigitList ),
	append( DigitList, [95, 95], NewList1 ),
	append( [82, 101, 115, 95, 95], NewList1, NewAtomList ),
	name( Atom, NewAtomList ).

%*****************************************************************************

process_switch_addord( ["-addord"] ) :-
        !,
        assert( cmd_arg( addord, [] ) ),
        assert( cmd_arg( genord, [] ) ),
        writeln( '# *** option: addord' ).


%*****************************************************************************
%* - check whether ordering addition really needs to be performed
%*****************************************************************************

test_switches_addord :-
		% if formula contains no equality
	predaritylist( PredArityList ),
	\+ member( equal/_, PredArityList ),
	!,
		% then remove all ordering additions
	retract_all(cmd_arg(addord,_)).

test_switches_addord.
