%###########################################################################
%# mod_brandstemod.pl
%# - STE modification of [Brand:SIAM;75]
%###########################################################################
%# predicates to be used from outside:
%#	- addrefleq/4
%#	- smod/4
%#	- tmod/4
%#	- emod/4
%#	- test_switches_stemod/0
%#	- process_switch_stemod/1
%# dependent on
%#	- mylib.pl
%#	- clause info facts generated by siginfo.pl
%###########################################################################
%#        Joachim Steinbach               Automated Reasoning Research Group
%# Email: steinbac@informatik.tu-muenchen.de  Lehrstuhl VIII f"ur Informatik
%# Tel.:  +49-89/289-27913		    Technische Universit"at M"unchen
%# Fax.:  +49-89/289-27902			   D-80290 M"unchen, Germany
%# http://wwwjessen.informatik.tu-muenchen.de/~steinbac
%###########################################################################


%*****************************************************************************
%* addrefleq( problemwise, +ClauseList, +SwitchList, -NewClauseList )
%*****************************************************************************
%* - just add equal(X,X) as first clause to clause list
%*****************************************************************************

b_addrefleq( problemwise, ClauseList, _, [ReflEq|ClauseList] ) :-
		% just construct reflexivity axiom
	ReflEq = rule( reflexivity, axiom, [++equal(X,X)], [], ['X'=X] ).

%*****************************************************************************


%*****************************************************************************
%* smod( clausewise, +Clause, +SwitchList, -NewClauseList )
%*****************************************************************************
%* - perform s-modification
%*****************************************************************************

b_smod( clausewise, Clause, _, [Clause] ) :-
		% don't modify equality clauses
	Clause = clause( _, _, LitList, _, _ ),
	unwanted_equality_axiom( LitList, [r,s,t,e] ),
	!.

b_smod( clausewise, Clause, _, NewClauseList ) :-
	Clause = clause( Name, Type, LitList, Cons, Dict ),
	!,
	b_s_modif( LitList, NewLitListList ),
	addinfo2newclauselist( Name, Type, NewLitListList, Cons, Dict, 
								NewClauseList ).

b_smod( clausewise, Clause, _, [Clause] ).

%*****************************************************************************


%*****************************************************************************
%* tmod( clausewise, +Clause, +SwitchList, -NewClauseList )
%*****************************************************************************
%* - perform t-modification
%*****************************************************************************

b_tmod( clausewise, Clause, _, [Clause] ) :-
		% don't modify equality clauses
	Clause = clause( _, _, LitList, _, _ ),
	unwanted_equality_axiom( LitList, [r,s,t,e] ),
	!.

b_tmod( clausewise, Clause, _, [NewClause] ) :-
	Clause = clause( Name, Type, LitList, Cons, Dict ),
	!,
	b_t_modif( LitList, NewTmpLitList, PreDict ),
	b_predict2dict( PreDict, Dict, NewVarDict ),
	append( Dict, NewVarDict, NewDict ),
	b_gen_companion_clause( NewTmpLitList, NewLitList ),
	NewClause = clause( Name, Type, NewLitList, Cons, NewDict ).

b_tmod( clausewise, Clause, _, [Clause] ).

%*****************************************************************************


%*****************************************************************************
%* emod( clausewise, +Clause, +SwitchList, -NewClauseList )
%*****************************************************************************
%* - perform e-modification
%*****************************************************************************

b_emod( clausewise, Clause, _, [Clause] ) :-
		% don't modify equality clauses
	Clause = clause( _, _, LitList, _, _ ),
	unwanted_equality_axiom( LitList, [r,s,t,e] ),
	!.

b_emod( clausewise, Clause, _, [NewClause] ) :-
	Clause = clause( Name, Type, LitList, Cons, Dict ),
	!,
	b_e_modif( LitList, NewTmpLitList, PreDict ),
	b_predict2dict( PreDict, Dict, NewVarDict ),
	append( Dict, NewVarDict, NewDict ),
	b_gen_companion_clause( NewTmpLitList, NewLitList ),
	NewClause = clause( Name, Type, NewLitList, Cons, NewDict ).

b_emod( clausewise, Clause, _, [Clause] ).

%*****************************************************************************


%*****************************************************************************
%* s_modif( +Clause, -S_ClauseList )
%*****************************************************************************
%* S-modification: introduce symmetrical positive equations
%*****************************************************************************

b_s_modif( [++equal(L,R)|RestClause], S_ClauseList ) :-
	L == R,
	!,
	b_s_modif( RestClause, S_RestClauseList ),
	prependtoall( ++equal(L,R), S_RestClauseList, S_ClauseList ).

b_s_modif( [++equal(L,R)|RestClause], S_ClauseList ) :-
	!,
	b_s_modif( RestClause, S_RestClauseList ),
	prependtoall( ++equal(L,R), S_RestClauseList, PrependL ),
	prependtoall( ++equal(R,L), S_RestClauseList, PrependR ),
	append( PrependL, PrependR, S_ClauseList ).

b_s_modif( [Literal|RestClause], S_ClauseList ) :-
	b_s_modif( RestClause, S_RestClauseList ),
	prependtoall( Literal, S_RestClauseList, S_ClauseList ).

b_s_modif( [], [[]] ).
	
%*****************************************************************************


%*****************************************************************************
%* t_modif( +Clause, -T_ClauseList, -T_PreDict )
%*****************************************************************************
%* - compile transitivity
%*****************************************************************************

b_t_modif( [], [], [] ).

b_t_modif( [++equal(L,R)|RestClause], [++equal(L,(NewSubterm->R))|T_RestClause], [(R,Ref)|PreDict] ) :-
	ext_var( R ),
	R =.. [Funct,_,S],
	!,
        NewSubterm =.. [Funct,Ref,S],
	getnewvar( Ref ),
	b_t_modif( RestClause, T_RestClause, PreDict ).

b_t_modif( [++equal(L,R)|RestClause], [++equal(L,(Ref->R))|T_RestClause], [(R,Ref)|PreDict] ) :-
	ext_var( R ),
	!,
	getnewvar( Ref ),
	b_t_modif( RestClause, T_RestClause, PreDict ).

% integrated, 17.02.98, Joachim
b_t_modif( [++equal(L,R)|RestClause], [++equal(L,(NewSubterm->R))|T_RestClause], 
						[(Funct,Ref)|PreDict]) :-
	R =.. [Funct,Subterm,S],
        cmd_arg(sort,Sorts),
        member(Funct,Sorts),
        NewSubterm =.. [Funct,Ref,S],
	getnewvar( Ref ),
	b_t_modif( RestClause, T_RestClause, PreDict ),
        !.

b_t_modif( [++equal(L,R)|RestClause], [++equal(L,(Ref->R))|T_RestClause], 
						[(Funct,Ref)|PreDict]) :-
	!,
	R =.. [Funct|_],
	getnewvar( Ref ),
	b_t_modif( RestClause, T_RestClause, PreDict ).

b_t_modif( [--equal(L,R)|RestClause], [--equal(L,R)|T_RestClause], PreDict ) :-
	!,
	b_t_modif( RestClause, T_RestClause, PreDict ).

b_t_modif( [Literal|RestClause], [Literal|T_RestClause], PreDict ) :-
	b_t_modif( RestClause, T_RestClause, PreDict ).

%*****************************************************************************


%*****************************************************************************
%* e_modif( +Clause, -E_Clause, -PreDict )
%*****************************************************************************
%* E-Modification: Flattening
%*****************************************************************************

b_e_modif( LitList, ModLitList, PreDict ) :-
	b_e_modif_literal_list( LitList, ModLitList, PreDict ).


b_e_modif_literal_list( [], [], [] ).

b_e_modif_literal_list( [Lit|RestLiterals], ModLiterals, PreDict ) :-
	b_e_modif_literal( Lit, ModLitList, PreDictLit ),
	b_e_modif_literal_list( RestLiterals, ModRestLiterals, PreDictRest ),
	append( ModLitList, ModRestLiterals, ModLiterals ),
	append( PreDictLit, PreDictRest, PreDict ).


b_e_modif_literal( ++equal(L,R), [++equal(ModL,ModR)], PreDict ) :-
	!,
	b_e_modif_root_term( L, ModL, PreDictL ),
	b_e_modif_root_term( R, ModR, PreDictR ),
	append( PreDictL, PreDictR, PreDict ).

b_e_modif_literal( --equal(L,R), [--equal(L,R)], [] ) :-
        ext_var(L), ext_var(R),
        !.

b_e_modif_literal( --equal(L,R), [--equal(ModL,ModR)], PreDict ) :-
	!,
	b_e_modif_root_term( L, ModL, PreDictL ),
	b_e_modif_root_term( R, ModR, PreDictR ),
	append( PreDictL, PreDictR, PreDict ).

b_e_modif_literal( ++Literal, [++ModLiteral], PreDict ) :-
	b_e_modif_root_term( Literal, ModLiteral, PreDict ).

b_e_modif_literal( --'$lpo'(L,R,Res), [--'$lpo'(L,R,Res)], [] ) :-
	!.

b_e_modif_literal( --Literal, [--ModLiteral], PreDict ) :-
	b_e_modif_root_term( Literal, ModLiteral, PreDict ).


b_e_modif_root_term( Term, ModTerm, RealPreDict ) :-
	b_e_modif_term( Term, PreModTerm, PreDict ),
	( ext_var(PreModTerm) ->
		( ModTerm=PreModTerm, RealPreDict=[]);
		(PreModTerm=(_->ModTerm),PreDict=[_|RealPreDict])
	).

b_e_modif_term( Var, Var, [] ) :-
	ext_var( Var ),
	!.

% integrated, 17.02.98, Joachim
b_e_modif_term( Term, (NewLeft -> NewRight), [(Functor,Ref)|PreDict] ) :-
	Term =.. [Funct,Subterm,S],
        cmd_arg(sort,Sorts),
        member(Funct,Sorts),
        Subterm =.. [Functor|Args],
	b_e_modif_term_list( Args, EModArgs, PreDict ),
	EModTerm =.. [Functor|EModArgs],
	getnewvar( Ref ),
        NewLeft =.. [Funct,Ref,S],
        NewRight =.. [Funct,EModTerm,S],
        !.

b_e_modif_term( Term, (Ref -> EModTerm), [(Functor,Ref)|PreDict] ) :-
	Term =.. [Functor|Args],
	b_e_modif_term_list( Args, EModArgs, PreDict ),
	EModTerm =.. [Functor|EModArgs],
	getnewvar( Ref ).


b_e_modif_term_list( [], [], [] ).

b_e_modif_term_list( [Var|RestArgs], [Var|EModRestArgs], PreDict ) :-
	ext_var( Var ),
	!,
	b_e_modif_term_list( RestArgs, EModRestArgs, PreDict ).

b_e_modif_term_list( [Arg|RestArgs], [EModArg|EModRestArgs], PreDict ) :-
	b_e_modif_term( Arg, EModArg, PreDict1 ),
	b_e_modif_term_list( RestArgs, EModRestArgs, PreDictRest ),
	append( PreDict1, PreDictRest, PreDict).


b_e_modif_list( [], [], [] ).

b_e_modif_list( [Clause|Rest], [E_Clause|E_Clauses_Rest], PreDict ) :-
	b_e_modif( Clause, E_Clause, ClausePreDict ),
	b_e_modif_list( Rest, E_Clauses_Rest, RestPreDict ),
	append( ClausePreDict, RestPreDict, PreDict ).

%*****************************************************************************


%*****************************************************************************
%* predict2dict( +PreDict, +RelDict, -Dict )
%*****************************************************************************
%* convert a list of pairs (Functor,NewVar) to a dictionary wrt. RelDict
%*****************************************************************************

b_predict2dict( [], _, [] ).

b_predict2dict( [(Funct,Ref)|PreDictRest], RelDict, 
					[UniqueArgSymb=Ref|NewVarDict] ) :-
	b_gen_unique_arg_symb( Funct, RelDict, 0, UniqueArgSymb ),
	b_predict2dict( PreDictRest, [UniqueArgSymb=Ref|RelDict], NewVarDict ).
	
b_gen_unique_arg_symb( ArgSymb, Dict, Number, UniqueArgSymb ) :-
	gen_corr_uc_symbol( ArgSymb, Number, NewArgSymb ),
	( member( (NewArgSymb=_), Dict ) ->
		( NextNumber is Number+1,
		  gen_unique_arg_symb( ArgSymb, Dict, NextNumber, UniqueArgSymb )
		);
		UniqueArgSymb = NewArgSymb
	).

%*****************************************************************************


%*****************************************************************************
%* gen_companion_clause( +STE_Clause, +STE_Modified_CLause )
%*****************************************************************************
%* create --equal subgoals for Var->Term links
%*****************************************************************************

b_gen_companion_clause( [], [] ).

b_gen_companion_clause( [Literal|RestLiterals], ModAllLiteralls ) :-
	b_gen_companion_lits( Literal, ModLiteral ),
	b_gen_companion_clause( RestLiterals, ModRestLiterals ),
	append( ModLiteral, ModRestLiterals, ModAllLiteralls ).


b_gen_companion_lits( ++Lit, [++ModLit|STE_Args] ) :-
	Lit =.. [Functor|Args],
	b_gen_companion_term_list( Args, ModArgs, STE_Args ),
	ModLit =.. [Functor|ModArgs].

b_gen_companion_lits( --Lit, [--ModLit|STE_Args] ) :-
	Lit =.. [Functor|Args],
	b_gen_companion_term_list( Args, ModArgs, STE_Args ),
	ModLit =.. [Functor|ModArgs].


b_gen_companion_term_list( [], [], [] ).

b_gen_companion_term_list( [Term|RestTerms], [ModTerm|RestModTerm], All_STE_Terms ) :-
	b_gen_companion_term( Term, ModTerm, STE_Terms ),
	b_gen_companion_term_list( RestTerms, RestModTerm, Rest_STE_Terms ),
	append( STE_Terms, Rest_STE_Terms, All_STE_Terms ).


b_gen_companion_term( Var, Var, [] ) :-
	ext_var( Var ),
	!.

b_gen_companion_term( (Var->Term), Var, NewSTE_Terms ) :-
	!,
	b_gen_companion_term( Term, ModTerm, STE_Terms ),
	append(STE_Terms,[--equal(ModTerm,Var)],NewSTE_Terms).

b_gen_companion_term( Term, ModTerm, STE_Terms ) :-
	Term =.. [Functor|Args],
	b_gen_companion_term_list( Args, ModArgs, STE_Terms ),
	ModTerm =.. [Functor|ModArgs].

%*****************************************************************************


%*****************************************************************************
%* process_switch_stemod( +ArrangedSwitch )
%*****************************************************************************
%* - what is to be done if -stemod is found
%*****************************************************************************

process_switch_brandstemod( ["-brandstemod"] ) :-
	!,
	assert( cmd_arg(b_addrefleq,[]) ),
	assert( cmd_arg(b_smod,[]) ),
	assert( cmd_arg(b_tmod,[]) ),
	assert( cmd_arg(b_emod,[]) ),
	writeln( '# *** option: brandstemod' ).

process_switch_brandstemod( ["-brandstemod"|Arguments] ) :-
	( member('s',Arguments) ->
		assert( cmd_arg(b_smod,[]) )
		;
		true
	),
	( member('t',Arguments) ->
		( assert( cmd_arg(b_tmod,[]) ), assert( cmd_arg(b_addrefleq,[]) ) )
		;
		true
	),
	( member('e',Arguments) ->
		( assert( cmd_arg(b_emod,[]) ), assert( cmd_arg(b_addrefleq,[]) ) )
		;
		true
	),
	write( '# *** option: brandstemod(' ),
	write( Arguments ),
	writeln( ')' ).

%*****************************************************************************


%*****************************************************************************
%* test_switches_stemod
%*****************************************************************************
%* - check whether ste-modification really needs to be performed
%*****************************************************************************

test_switches_brandstemod :-
		% if formula contains no equality
	predaritylist( PredArityList ),
	\+ member( equal/_, PredArityList ),
	!,
		% then remove all equality modifications
	retract_all(cmd_arg(b_addrefleq,_)),
	retract_all(cmd_arg(b_smod,_)),
	retract_all(cmd_arg(b_tmod,_)),
	retract_all(cmd_arg(b_emod,_)).

test_switches_brandstemod :-
		% if brandstemod was called but no rmeqax
	( cmd_arg(b_emod,_) ->
		L1 = [e] ; L1 = []
	),
	( cmd_arg(b_tmod,_) ->
		L2 = [t|L1] ; L2 = L1
	),
	( cmd_arg(b_smod,_) ->
		L3 = [s|L2] ; L3 = L2
	),
	( cmd_arg(b_addrefleq,_) ->
		L4 = [r|L3] ; L4 = L3
	),
	L4 \== [],
	\+ cmd_arg(rmeqax,_),
	!,
		% then remember to remove all equality axioms
	assert( cmd_arg(rmeqax,L4) ).

test_switches_brandstemod.

%*****************************************************************************
