%###########################################################################
%# mod_sorts.pl
%# ext_var:
%#          - checks whether a term is a variable X or a variable
%#            <op>(X,<sort>) including sort information (by using a sort
%#            operator <op>)
%# process_switch_sort:
%#          - asserts the sort operators
%###########################################################################
%###########################################################################
%#        Joachim Steinbach                Automated Reasoning Research Group
%# Email: steinbac@informatik.tu-muenchen.de   Lehrstuhl VIII f"ur Informatik
%# Tel.:  +49-89/289-27913		     Technische Universit"at M"unchen
%# Fax.:  +49-89/289-27902		            D-80290 M"unchen, Germany
%# http://wwwjessen.informatik.tu-muenchen.de/~steinbac
%###########################################################################

% integrated, 12.02.98, Joachim
ext_var( Lit ) :-
     var( Lit ),
     !.

ext_var( Lit ) :-
     Lit =.. [SOp,_,Snd],
     cmd_arg(sortfst,Sorts),
     member(SOp,Sorts),
     !,
     var( Snd ).

ext_var( Lit ) :-
     Lit =.. [SOp,Fst,_],
     cmd_arg(sortsnd,Sorts),
     member(SOp,Sorts),
     !,
     var( Fst ).


% integrated, 17.02.98, Joachim
process_switch_sort( ["-sortfst", T] ) :-
	cmd_arg( tptpout, _ ), !,
	retract( cmd_arg(sortfst,S) ),
        !,
	assert( cmd_arg(sortfst,[T|S]) ),
	write( '% *** option: sortfst(' ),
	write( T ),
	writeln( ')' ).

% integrated, 12.02.98, Joachim
process_switch_sort( ["-sortfst", S] ) :-
	cmd_arg( tptpout, _ ), !,
	assert( cmd_arg(sortfst,[S]) ),
	write( '% *** option: sortfst(' ),
	write( S ),
	writeln( ')' ).

% integrated, 06.04.98, Joachim
process_switch_sort( ["-sortsnd", T] ) :-
	cmd_arg( tptpout, _ ), !,
        retract( cmd_arg(sortsnd,S) ),
        !,
	assert( cmd_arg(sortsnd,[T|S]) ),
	write( '% *** option: sortsnd(' ),
	write( T ),
	writeln( ')' ).

% integrated, 06.04.98, Joachim
process_switch_sort( ["-sortsnd", S] ) :-
	cmd_arg( tptpout, _ ), !,
	assert( cmd_arg(sortsnd,[S]) ),
	write( '% *** option: sortsnd(' ),
	write( S ),
	writeln( ')' ).

process_switch_sort( ["-sortfst", T] ) :-
        retract( cmd_arg(sortfst,S) ),
        !,
	assert( cmd_arg(sortfst,[T|S]) ),
	write( '# *** option: sortfst(' ),
	write( T ),
	writeln( ')' ).

% integrated, 12.02.98, Joachim
process_switch_sort( ["-sortfst", S] ) :-
	assert( cmd_arg(sortfst,[S]) ),
	write( '# *** option: sortfst(' ),
	write( S ),
	writeln( ')' ).

% integrated, 06.04.98, Joachim
process_switch_sort( ["-sortsnd", T] ) :-
        retract( cmd_arg(sortsnd,S) ),
        !,
	assert( cmd_arg(sortsnd,[T|S]) ),
	write( '# *** option: sortsnd(' ),
	write( T ),
	writeln( ')' ).

% integrated, 06.04.98, Joachim
process_switch_sort( ["-sortsnd", S] ) :-
	assert( cmd_arg(sortsnd,[S]) ),
	write( '# *** option: sortsnd(' ),
	write( S ),
	writeln( ')' ).


% integrated, 03.03.98, Joachim
term_without_sorts( Term, Term ) :-
        var( Term ),
        !.

term_without_sorts( Term, NewTerm ) :-
        Term =.. [Funct,_,Subterm],
        cmd_arg( sortfst, Sorts ),
        member( Funct, Sorts ),
        !,
        term_without_sorts( Subterm, NewTerm ).

term_without_sorts( Term, NewTerm ) :-
        Term =.. [Funct,Subterm,_],
        cmd_arg( sortsnd, Sorts ),
        member( Funct, Sorts ),
        !,
        term_without_sorts( Subterm, NewTerm ).

term_without_sorts( Term, NewTerm ) :-
        Term =.. [Funct|Args],
        termlist_without_sorts( Args, NewArgs ),
        NewTerm =.. [Funct|NewArgs].


termlist_without_sorts( [Term|RestList], [NewTerm|NewRestList] ) :-
        term_without_sorts( Term, NewTerm ),
        termlist_without_sorts( RestList, NewRestList ).

termlist_without_sorts( [], [] ).


%###########################################################################
%# mod_noquerytrans.pl
%# process_switch_noquerytrans:
%#          - asserts the noquerytrans option
%###########################################################################
%###########################################################################
%#        Joachim Steinbach                Automated Reasoning Research Group
%# Email: steinbac@informatik.tu-muenchen.de   Lehrstuhl VIII f"ur Informatik
%# Tel.:  +49-89/289-27913		     Technische Universit"at M"unchen
%# Fax.:  +49-89/289-27902		            D-80290 M"unchen, Germany
%# http://wwwjessen.informatik.tu-muenchen.de/~steinbac
%###########################################################################

% integrated, 06.04.98, Joachim
process_switch_noquerytrans( ["-noquerytrans"] ) :-
	cmd_arg( tptpout, _ ), !,
    assert( cmd_arg( noquerytrans, [] ) ),
    writeln( '% *** option: noquerytrans' ).

process_switch_noquerytrans( ["-noquerytrans"] ) :-
    assert( cmd_arg( noquerytrans, [] ) ),
    writeln( '# *** option: noquerytrans' ).


%###########################################################################
%# mod_psetheo.pl
%# process_switch_psetheo
%#          - asserts the psetheo option
%###########################################################################
%###########################################################################
%#        Joachim Steinbach                Automated Reasoning Research Group
%# Email: steinbac@informatik.tu-muenchen.de   Lehrstuhl VIII f"ur Informatik
%# Tel.:  +49-89/289-27913		     Technische Universit"at M"unchen
%# Fax.:  +49-89/289-27902		            D-80290 M"unchen, Germany
%# http://wwwjessen.informatik.tu-muenchen.de/~steinbac
%###########################################################################

% integrated, 07.04.98, Joachim
process_switch_psetheo( ["-psetheo"] ) :-
	cmd_arg( tptpout, _ ), !,
    assert( cmd_arg( psetheo, [] ) ),
    writeln( '% *** option: psetheo' ).

process_switch_psetheo( ["-psetheo"] ) :-
    assert( cmd_arg( psetheo, [] ) ),
    writeln( '# *** option: psetheo' ).

